<?php
	require_once $twatch->path( 'db/DbPassiveDict.php' );

	abstract class TwatchEntityPassiveGene {
		const ERROR_ALREADY_EXISTS = 1;
		const ERROR_BAD_USER_STRING = 2;
		
		const MODE_NORMAL = 0;
		const MODE_ADD_ONLY = 1;
		const MODE_READ_ONLY = 2;
		
		const CONTEXT_IMPORT = 1;
		const CONTEXT_EXPLICIT_ADD = 2;
		const CONTEXT_API = 3;
		
		public $dict;
		
		public $mode;
		public $context;
		
		public function __construct( TwatchDbPassiveDict $dict, $mode = 0, $context = 0 ) {
			$this->dict = $dict;
			$this->mode = $mode;
			$this->context = $context;
		}
		
		public function getStringEntityVId( $string, TwatchWebsite $website = null  ) {
			throw new TwatchException( 'not implemented' );
		}
		
	}
	
	class TwatchEntDictPasvGene extends TwatchEntityPassiveGene {
		protected $dictId;
		
		public function __construct( $dictId, TwatchDbPassiveDict $dict, $mode = 0, $context = 0 ) {
			parent::__construct( $dict, $mode, $context );
			$this->dictId = $dictId;
		}
		
		public function getStringEntityVId( $string, TwatchWebsite $website = null  ) {
			$string = $this->prepString( $string, $website );
			return $this->getStringDictId( $string );
		}

		protected function prepString( $string, TwatchWebsite $website = null  ) {
			return $string;
		}

		protected function getStringDictId( $string ) {
			$id = $this->dict->getId( $this->dictId, $string );
			if( $id === false ) {
				if( $this->mode == self::MODE_READ_ONLY ) return false;
				return $this->dict->putString( $this->dictId, $string, array(), '', null, 0 );
			} elseif( $this->mode == self::MODE_ADD_ONLY ) {
				throw new TwatchUserError( 'Already Exists', 1 );
			}
			return $id;
		}
		
		
	}
	
	class TwatchEntIpPasvGene extends TwatchEntDictPasvGene {

		protected function prepString( $string, TwatchWebsite $website = null  ) {
			$ipStr = ardeGetIp( $string );
			if( $ipStr === false ) throw new ArdeUserError( "invalid Ip address '".$string."'", self::ERROR_BAD_USER_STRING );
			return $ipStr;
		}
		
		protected function getStringDictId( $string ) {
			$id = $this->dict->getId( $this->dictId, $string );
			if( $id === false ) {
				if( $this->mode == self::MODE_READ_ONLY ) return false;
				$ipId = ardeIpToU32( $string );
				return $this->dict->putString( $this->dictId, $string, array(), '', $ipId, 0 );
			} elseif( $this->mode == self::MODE_ADD_ONLY ) {
				throw new TwatchException( 'Already Exists', 1 );
			}
			return $id;
		}

	}
	
	class TwatchEntPagePasvGene extends TwatchEntDictPasvGene {
		public function __construct( TwatchDbPassiveDict $dict, $mode = 0, $context = 0 ) {
			parent::__construct( TwatchDict::PAGE, $dict, $mode, $context );
		}
		protected function prepString( $string, TwatchWebsite $website = null ) {
			return $website->getId().'-'.$string;
		}
	}
	
	class TwatchEntRefGroupPasvGene extends TwatchEntDictPasvGene {
		protected $searchENameIds = array();
		
		public function __construct( TwatchDbPassiveDict $dict, $mode = 0, $context = 0 ) {
			global $twatch;
			parent::__construct( TwatchDict::REF_DOMAIN, $dict, $mode, $context );
			foreach( $twatch->config->getList( TwatchConfig::SEARCH_ENGINES ) as $id => $searchEngine ) {
				$this->searchENameIds[ strtolower( $searchEngine->name ) ] = $id;
			}
			
		}
		
		public function getStringEntityVId( $string, TwatchWebsite $website = null  ) {
			if( !preg_match( '/^(.+?)\s?(?:\:\s?(.+)|)$/', $string, $matches ) ) {
				return false;
			}
			if( strtolower( $matches[1] ) == 'search engine' ) {
				if( isset( $this->searchENameIds[ strtolower( $matches[2] ) ] ) ) {
					return $this->searchENameIds[ strtolower( $matches[2] ) ];
				} else {
					return false;
				}
			} elseif( strtolower( $matches[1] ) == 'domain' ) {
				return $this->getStringDictId( $matches[2] );
			} elseif( strtolower( $matches[1] ) == 'other' ) {
				return TwatchEntGeneRefGroup::NONE;
			}
			return false;	
		}
	}
	
	class TwatchEntHourPasvGene extends TwatchEntityPassiveGene {
		public function getStringEntityVId( $string, TwatchWebsite $website = null  ) {
			$num = (int)$string;
			if( $num >= 0 && $num < 24 ) {
				return $num + 1;
			}
			return false;
		}
	}
	
	class TwatchEntWeekdayPasvGene extends TwatchEntityPassiveGene {
		public static $dayIds = array(
			'sunday' => 1, 'monday' => 2, 'tuesday' => 3, 'wednesday' => 4, 'thursday' => 5, 'friday' => 6, 'saturday' => 7
		);
		
		public function getStringEntityVId( $string, TwatchWebsite $website = null  ) {
			if( isset( self::$dayIds[ strtolower( $string ) ] ) ) {
				return self::$dayIds[ strtolower( $string ) ];
			}
			return false;
		}
	}
	
	class TwatchEntUserAgentPasvGene extends TwatchEntityPassiveGene {
		protected $userAgentNameIds = array();
		
		public function __construct( TwatchDbPassiveDict $dict, $mode = 0, $context = 0 ) {
			global $twatch;
			parent::__construct( $dict, $mode, $context );
			foreach( $twatch->config->getList( TwatchConfig::USER_AGENTS ) as $id => $userAgent ) {
				$name = strtolower( $userAgent->name );
				if( preg_match( '/^(.+)\srobot$/', $name, $matches ) ) $name = $matches[1];
				$this->userAgentNameIds[ $name ] = $id; 
			}
		}
		
		public function getStringEntityVId( $string, TwatchWebsite $website = null  ) {
			$string = strtolower( $string );
			if( preg_match( '/^(.+)\srobot$/', $string, $matches ) ) $string = $matches[1];
			if( isset( $this->userAgentNameIds[ $string ] ) ) {
				return $this->userAgentNameIds[ $string ];
			} else {
				return false;
			}
		}
	}
	
	$twatch->includePosition( 'passive_gene' );
?>