<?php
	/********************************************************************/
	/*                                                                  */
	/*      Copyright (C) 2004 Arash Dejkam, All Rights Reserved.       */
	/*      http://www.tracewatch.com                                   */
	/*                                                                  */
	/*      Please read the licence file distributed with this          */
	/*      file or the one available at tracewatch.com for             */
	/*      the terms under which you can use or modify this file.      */
	/*                                                                  */
	/********************************************************************/
	
	global $ardeBase;
	
	require_once $ardeBase->path( 'lib/General.php' );
	require_once $ardeBase->path( 'lib/ArdeProperties.php' );
	require_once $ardeBase->path( 'lib/ArdeDisTaskManager.php' );
	require_once $ardeBase->path( 'lib/ArdeDbErrorReporter.php' );
	
	class TwatchTime extends ArdeTime {
		public static function getTime( $ts = 0 ) {
			return new self( $ts );
		}
		public function __clone() {
			return new self( $this->ts );
		}
		
		public function duplicate() {
			return new self( $this->ts );
		}

	}
	
	function twatchSetAppTime() {
		global $twatch;
		$twatch->now = new TwatchTime( time() + $twatch->config->get( TwatchConfig::TIME_DIFFERENCE ) );
	} 
	
	class TwatchException extends ArdeException {
		protected $class = "TraceWatch Error";
	}
	
	class TwatchWarning extends ArdeException {
		protected $class = "TraceWatch Warning";
		protected $type = ArdeException::WARNING;
	}
	
	
	class TwatchUserError extends TwatchException {
		protected $class = "TraceWatch User Error"; 
		protected $type = ArdeException::USER_ERROR;
	}

	class TwatchTaskManager extends ArdeDisTaskManager {
		public function __construct() {
			global $twatch;
			parent::__construct( $twatch->db, '', $twatch->now );
		}
	}
	
	
	class TwatchPropertyChangesDb extends ArdePropertyChangesDb {
		public function __construct( $db, $tableName ) {
			parent::__construct( $db, '', $tableName );
		}
		
		protected function getExtraKeyValues() {
			return array();
		}
	}
	
	abstract class TwatchProperties extends ArdeAdminProperties {
		const USER_LAYER = 1000;
		
		const IDS_PER_LAYER = 500000;
		
		protected $db;
		
		function __construct( ArdeDb $db = null ) {
			$this->db = $db;
			$this->changes = new ArdePropertyChanges();
		}
		
		public function getLayerStartId( $layer ) {
			return $layer * self::IDS_PER_LAYER;
		}
		
		protected function applyUserChanges() {
			$this->advanceLayerTo( self::USER_LAYER );
			$changes = $this->changes->getChanges( array_keys( $this->data ) );
			$this->applyChanges( $changes );
		}
		
		public function applyAllChanges() {
			$this->applyPluginChanges();
			$this->applyUserChanges();
			
		}
		
		abstract protected function getTableName();
		
		public function advanceLayerTo( $layer ) {
			if( $layer == $this->layer ) return;
			if( $layer == self::USER_LAYER ) {
				$this->changes =  new TwatchPropertyChangesDb( $this->db, $this->getTableName() );
			}
			parent::advanceLayerTo( $layer );
		}
		
		public function install( $overwrite ) {
			$changes = new TwatchPropertyChangesDb( $this->db, $this->getTableName() );
			$changes->install( $overwrite );
		}
		
		public function uninstall() {
			$changes = new TwatchPropertyChangesDb( $this->db, $this->getTableName() );
			$changes->uninstall();
		}
	}
	
	class TwatchConfig extends TwatchProperties {

		public static $defaultProperties;
		
		const COUNTERS = 1;
		const ENTITIES = 2;
		const WEBSITES = 3;
		const RDATA_WRITERS = 4;
		const STATS_PAGES = 5;
		const PATH_ANALYZER = 6;
		const DICTS = 7;
		const LATEST_PAGE = 8;
		const VISITOR_TYPES = 9;
		const LATEST = 10;
		const USER_AGENTS = 19;
		const SEARCH_ENGINES = 20;
		const TIME_DIFFERENCE = 21;
		const TIME_ZONE_NAME = 22;
		const COOKIE_KEYS = 23;
		const ADMIN_COOKIE = 24;
		const DEFAULT_LANG = 25;
		const LOGGER_WHEN = 26;
		const COUNTERS_WHEN = 27;
		const VERSION = 28;
		const PLUGIN_VERSIONS = 29;
		
		public static $idStrings = array(
			 self::COUNTERS => 'Counters'
			,self::ENTITIES => 'Entities'
			,self::WEBSITES => 'Websites'
			,self::RDATA_WRITERS => 'RData Writers'
			,self::STATS_PAGES => 'Stats Pages'
			,self::PATH_ANALYZER => 'Path Analyzer'
			,self::DICTS => 'Dicts'
			,self::LATEST_PAGE => 'Latest Page'
			,self::VISITOR_TYPES => 'Visitor Types'
			,self::LATEST => 'Latest'
			,self::USER_AGENTS => 'User Agents'
			,self::SEARCH_ENGINES => 'Search Engines'
			,self::COOKIE_KEYS => 'Cookie Keys'
			,self::LOGGER_WHEN => 'Logger When'
			,self::COUNTERS_WHEN => 'Counters When'
		);
		
		public function getIdString( $id ) {
			if( !isset( TwatchConfig::$idStrings[ $id ] ) ) return $id;
			return $id.' ('.TwatchConfig::$idStrings[ $id ].')';
		}
		
		protected function getTableName() { return 'c'; }

		protected function applyPluginChanges() {
			global $twatch;
			foreach( $twatch->plugins as $plugin ) {
				$this->advanceLayerTo( $plugin->getLayer() );
				$plugin->startId = $this->getStartId();
				$plugin->applyConfigChanges( $this, $this->data );
			}
			
		}
		
		

	}
	
	class TwatchState extends TwatchProperties {
		
		public static $defaultProperties;
		
		const COUNTERS_AVAIL = 2;
		const PATH_NEXT_CLEANUP_ROUND = 3;
		const PATH_ANALYZER_INSTALLED = 4;
		const OFF_ENTITY =  5;
		const DICT_STATES =  6;
		const OFF_COUNTER = 7;
		const USER_AGENT_CACHE_VALID = 8;
		const SEARCHE_CACHE_VALID = 9;
		
		public static $extraDefaults;
		
		public static $idStrings = array(
			 self::COUNTERS_AVAIL => 'Counters Avail'
			,self::PATH_NEXT_CLEANUP_ROUND => 'Path Next Cleanup Round'
			,self::PATH_ANALYZER_INSTALLED => 'Path Analyzer Installed'
			,self::OFF_ENTITY =>  'Off Entity'
			,self::DICT_STATES =>  'Dict States'
		);
		
		public function getIdString( $id ) {
			if( !isset( TwatchState::$idStrings[ $id ] ) ) return $id;
			return $id.' ('.TwatchState::$idStrings[ $id ].')';
		}
		
		protected function getTableName() { return 's'; }
		
		protected function applyPluginChanges() {
			global $twatch;
			foreach( $twatch->plugins as $plugin ) {
				$this->advanceLayerTo( $plugin->id );
				$plugin->startId = $this->getStartId();
				$plugin->applyStateChanges( $this, $this->data );
			}
			
		}
		
	}
	
	class TwatchUserDataChanges extends ArdePropertyChangesDb {
		public $userId;
		
		public function __construct( $db, $userId ) {
			$this->userId = $userId;
			parent::__construct( $db, '', 'ud', array( 'uid' ) );
		}
		
		protected function getExtraKeyValues() {
			return array( 'uid' => $this->userId );
		}
		
		
	}
	
	class TwatchUserData extends ArdeAdminProperties {
		
		const VIEW_REPORTS = 1;
		const VIEW_ADMIN = 2;
		const ADMINISTRATE = 3;
		const VIEW_ERRORS = 4;
		const KEYS = 5;
		const VIEW_IPS = 6;
		const VIEW_COOKIE_IDS = 7;
		const VIEW_ADMIN_IN_LATEST = 8;
		const VIEW_PRIVATE_COMMENTS = 9;
		
		public static $defaultProperties;
		
		public static $idStrings = array(
			 self::VIEW_REPORTS => 'View Reports'
			,self::VIEW_ADMIN => 'View Admin'
			,self::ADMINISTRATE => 'Administrate'
		);
		
		public function getLayerStartId( $layer ) {
			return 500000;
		}
		
		public function getIdString( $id ) {
			if( !isset( self::$idStrings[ $id ] ) ) return $id;
			return $id.' ('.self::$idStrings[ $id ].')';
		}
		
		public function __construct( $userId ) {
			global $twatch;
			parent::__construct();
			$this->changes = new TwatchUserDataChanges( $twatch->db, $userId );
		}
		
		public function install( $overwrite ) {
			$this->changes->install( $overwrite );
		}
		
		public function uninstall() {
			$this->changes->uninstall();
		}
		
		public function loadChanges() {
			$changes = $this->changes->getChanges( array_keys( $this->data ) );
			$this->applyChanges( $changes );
		}
		
		
	}
	
	
	
	
	
	
	class TwatchErrorLogger extends ArdeDbErrorReporter {
		public function __construct( ArdeErrorReporter $secondReporter = null, $storeRestrictions = false ) {
			global $twatch;
			parent::__construct( $twatch->db, '', 'err', $secondReporter, $storeRestrictions );
		}
	}
	
	
	
	$twatch->applyOverrides( array( 'twatchSetAppTime' => true ) );
	
?>