<?php
	/********************************************************************/
	/*                                                                  */
	/*      Copyright (C) 2004 Arash Dejkam, All Rights Reserved.       */
	/*      http://www.tracewatch.com                                   */
	/*                                                                  */
	/*      Please read the licence file distributed with this          */
	/*      file or the one available at tracewatch.com for             */
	/*      the terms under which you can use or modify this file.      */
	/*                                                                  */
	/********************************************************************/
    
	require_once $twatch->path( 'lib/EntityPassiveGene.php' );

	class TwatchImportFileInfo {
		public $name = '!NS!';
		public $counters;
		public $starts;
		public $stops;
		public $periodTypes;
		
		public $rowCount = 0;
		
		public function __construct( $starts, $stops, $counters, $periodTypes ) {
			$this->starts = $starts;
			$this->stops = $stops;
			$this->counters = $counters;
			$this->periodTypes = $periodTypes;
			$this->determineStartStopTs();
			$this->determineTotalPeriodsCount();
		}
		
		public function determineStartStopTs() {
			global $twatch;
			foreach( $this->periodTypes as $id => &$periodType ) {
				
				if( isset( $this->starts[ $id ] ) ) {
					$periodType->determinedStartTs = $this->starts[ $id ]->ts;
				} elseif( isset( $this->starts[ -1 ] ) ) {
					$periodType->determinedStartTs = $this->starts[ -1 ]->ts;
				} else {
					$min = TwatchPeriod::fromCode( $id, $periodType->minCode );
					$periodType->determinedStartTs = $min->getStartTs();
				}
				
				if( isset( $this->stops[ $id ] ) ) {
					$periodType->determinedStopTs = $this->stops[ $id ]->ts;
				} elseif( isset( $this->stops[ -1 ] ) ) {
					$periodType->determinedStopTs = $this->stops[ -1 ]->ts;
				} else {
					$max = TwatchPeriod::fromCode( $id, $periodType->maxCode );
					$periodType->determinedStopTs = $max->getEndTs();
				}
				if( $periodType->determinedStopTs > $twatch->now->ts ) $periodType->determinedStopTs = $twatch->now->ts;
				
			}
		}
		
		
		public function determineTotalPeriodsCount() {
			foreach( $this->counters as $counter ) {
				$this->rowCount += $counter->rowCount;
			}
		}
		
		public static function fromXml( DOMElement $element ) {
			global $twatch;
			
			$starts = array();
			foreach( new ArdeXmlElemIter( $element, 'start' ) as $startE ) {
				$periodType = ArdeXml::strAttribute( $startE, 'period_type', -1 );
				if( $periodType != -1 ) {
					if( isset( TwatchPeriod::$importStringTypes[ $periodType ] ) ) {
						$periodType = TwatchPeriod::$importStringTypes[ $periodType ];
					} else {
						throw new TwatchException( 'bad start period type "'.$periodType.'"' );
					}
				}

				$starts[ $periodType ] = TwatchImportTime::fromXml( $startE );
				$starts[ $periodType ]->periodType = $periodType;
			}
			
			$stops = array();
			foreach( new ArdeXmlElemIter( $element, 'stop' ) as $stopE ) {
				$periodType = ArdeXml::strAttribute( $stopE, 'period_type', -1 );
				if( $periodType != -1 ) {
					if( isset( TwatchPeriod::$importStringTypes[ $periodType ] ) ) {
						$periodType = TwatchPeriod::$importStringTypes[ $periodType ];
					} else {
						throw new TwatchException( 'bad start period type "'.$periodType.'"' );
					}
				}

				$stops[ $periodType ] = TwatchImportTime::fromXml( $stopE );
				$stops[ $periodType ]->periodType = $periodType;
			}
			
			$twCounters = $twatch->config->getList( TwatchConfig::COUNTERS );
			$counters = array();
			$periodTypes = array();
			$counterEs = new ArdeXmlElemIter( $element, 'counter' );
			foreach( $counterEs as $counterE ) {
				$counter = TwatchImportCounterInfo::fromXml( $counterE );
				
				foreach( $counter->periodTypes as $id => $periodType ) {
					if( !isset( $periodTypes[ $id ] ) ) $periodTypes[ $id ] = new TwatchPeriodTypeInfo( $id, null );
					if( $periodTypes[ $id ]->minCode === null || $periodTypes[ $id ]->minCode > $periodType->minCode ) $periodTypes[ $id ]->minCode = $periodType->minCode;
					if( $periodTypes[ $id ]->maxCode === null || $periodTypes[ $id ]->maxCode < $periodType->maxCode ) $periodTypes[ $id ]->maxCode = $periodType->maxCode; 
				}
				foreach( $twCounters as $twCounter ) {
					if( $twCounter->allowImport() && $twCounter->importNameMatches( $counter->name ) ) {
						$counter->mappedId = $twCounter->id;
						break;
					}
				}
				$counters[] = $counter;
			}
			
			return new self( $starts, $stops, $counters, $periodTypes );
		}	
		
		public function printXml( ArdePrinter $p, $tagName, $extraAttrib = '' ) {
			$p->pl( '<'.$tagName.' name="'.ardeXmlEntities( $this->name ).'"'.$extraAttrib.'>', 1 );
			foreach( $this->starts as $start ) {
				if( $start->periodType == -1 ) $pt = '';
				else $pt = ' period_type="'.$start->periodType.'"';
				$p->pl( '<start ts="'.$start->ts.'"'.$pt.'>'.ardeXmlEntities( $start->title ).'</start>' );
			}
			foreach( $this->stops as $stop ) {
				if( $stop->periodType == -1 ) $pt = '';
				else $pt = ' period_type="'.$stop->periodType.'"';
				$p->pl( '<stop ts="'.$stop->ts.'"'.$pt.'>'.ardeXmlEntities( $stop->title ).'</stop>' );
			}
			foreach( $this->periodTypes as $periodType ) {
				$periodType->printXml( $p, 'period_type' );
				$p->nl();
			}
			
			foreach( $this->counters as $counter ) {
				$counter->printXml( $p, 'counter' );
				$p->nl();
			}
			
			$p->rel();
			$p->pn( '</'.$tagName.'>' );
		}
	}
	
	class TwatchImportCounterInfo {
		public $name;
		public $mappedId = null;
		public $periodTypes;
		public $rowCount = 0;
		
		public function __construct( $name, $periodTypes, $element, $rowCount ) {
			$this->name = $name;
			$this->periodTypes = $periodTypes;
			$this->element = $element;
			$this->rowCount = $rowCount;
		}
		
		public static function fromXml( DOMElement $element ) {
			global $twatch;
			
			$name = ArdeXml::strAttribute( $element, 'name' );
			
			$rowCount = 0;

			$periodEs = new ArdeXmlMultiElemIter( $element, TwatchPeriod::getImportTagNames() );
			$periodTypes = array();
			foreach( $periodEs as $periodE ) {
				$period = TwatchPeriod::fromImportElement( $periodE );
				
				$count = 0;
				foreach( new ArdeXmlElemIter( $periodE, 'row' ) as $row ) ++$count;
				if( $count == 0 ) ++$rowCount;
				else $rowCount += $count;
				
				if( !isset( $periodTypes[ $period->type ] ) ) $periodTypes[ $period->type ] = new TwatchPeriodTypeInfo( $period->type, 1 );
				else ++$periodTypes[ $period->type ]->count;
				if( $periodTypes[ $period->type ]->minCode === null || $periodTypes[ $period->type ]->minCode > $period->getCode() ) {
					$periodTypes[ $period->type ]->minCode = $period->getCode();
				}
				if( $periodTypes[ $period->type ]->maxCode === null || $periodTypes[ $period->type ]->maxCode < $period->getCode() ) {
					$periodTypes[ $period->type ]->maxCode = $period->getCode();
				}
			}
			
			return new self( $name, $periodTypes, $element, $rowCount );
		}
		
		public function printXml( ArdePrinter $p, $tagName, $extraAttrib = '' ) {
			if( $this->mappedId === null ) {
				$mapped = '';
			} else {
				$mapped = ' mapped_id="'.$this->mappedId.'"';
			}
			$p->pl( '<'.$tagName.' name="'.ardeXmlEntities( $this->name ).'"'.$mapped.$extraAttrib.'>', 1 );
			foreach( $this->periodTypes as $periodType ) {
				$periodType->printXml( $p, 'period_type' );
				$p->nl();
			}
			
			$p->rel();
			$p->pn( '</'.$tagName.'>' );
		}
	}
	
	class TwatchPeriodTypeInfo {
		public $id;
		public $count;
		public $minCode = null;
		public $maxCode = null;
		public $determinedStartTs = null;
		public $determinedStopTs = null;
		
		public function __construct( $id, $count ) {
			$this->id = $id;
			$this->count = $count;
		}
		
		public function printXml( ArdePrinter $p, $tagName, $extraAttrib = '' ) {
			global $twatch;
			
			$min = TwatchPeriod::fromCode( $this->id, $this->minCode );
			$max = TwatchPeriod::fromCode( $this->id, $this->maxCode );
			
			if( $this->count === null ) {
				$count = '';
			} else {
				$count = ' count="'.$this->count.'"';
			}
			$p->pl( '<'.$tagName.' id="'.$this->id.'" name="'.TwatchPeriod::getTypeString( $this->id ).'"'.$count.$extraAttrib.'>' );
			$p->pl( '	<min code="'.$min->getCode().'" ts="'.$min->time->getDayStart().'">'.ardeXmlEntities( $min->getString( TwatchPeriod::STRING_IMPORT_NAME ) ).'</min>' );
			$p->pl( '	<max code="'.$max->getCode().'" ts="'.$max->time->dayOffset(1)->getDayStart().'">'.ardeXmlEntities( $max->getString( TwatchPeriod::STRING_IMPORT_NAME ) ).'</max>' );
			if( $this->determinedStartTs !== null ) {
				$p->pl( '	<det_start_ts>'.$this->determinedStartTs.'</det_start_ts>' );
			}
			if( $this->determinedStopTs !== null ) {
				$p->pl( '	<det_stop_ts>'.$this->determinedStopTs.'</det_stop_ts>' );
			}
			$p->pn( '</'.$tagName.'>' );			
		}
	}
	
	class TwatchImportTime {
		public $ts;
		public $title;
		public $periodType;
		
		public function __construct( $ts, $title, $periodType ) {
			$this->ts = $ts;
			$this->title = $title;
			$this->periodType = $periodType;
		}
		
		public static function fromXml( DOMElement $element ) {
			global $twatch;
			$str = ArdeXml::strContent( $element );
			if( !preg_match( '/(\d\d\d\d)\-(\w\w\w)\-(\d\d)\s(\d\d):(\d\d):(\d\d)\sGMT/', $str, $matches ) ) {
				throw new TwatchException( 'bad date string "'.$str."'" );
			}
			if( !isset( TwatchTime::$monthShortNums[ strtolower( $matches[2] ) ] ) ) throw new TwatchException( 'invalid month "'.$matches[2] );
			$month = TwatchTime::$monthShortNums[ strtolower( $matches[2] ) ];
			$ts = gmmktime( (int)$matches[4], (int)$matches[5], (int)$matches[6], $month, (int)$matches[3], (int)$matches[1] ) + $twatch->config->get( TwatchConfig::TIME_DIFFERENCE );
			$title = gmdate( 'Y-M-d H:i:s', $ts );
			return new self( $ts, $title, null );
		}
	}
	
?>