<?php
	/********************************************************************/
	/*                                                                  */
	/*      Copyright (C) 2004 Arash Dejkam, All Rights Reserved.       */
	/*      http://www.tracewatch.com                                   */
	/*                                                                  */
	/*      Please read the licence file distributed with this          */
	/*      file or the one available at tracewatch.com for             */
	/*      the terms under which you can use or modify this file.      */
	/*                                                                  */
	/********************************************************************/
    
	require_once $ardeUser->path( 'db/ArdeDbUser.php' );
	require_once $ardeBase->path( 'lib/General.php' );
	
	class ArdeUser {

		const USER_PUBLIC = 1;
		const USER_ROOT = 2;
		
		public $id;
		public $username;
		public $random;
		public $randomExpires;
		public $data;
		
		public function __construct( $id, $username ) {
			$this->id = $id;
			$this->username = $username;
		}
		
		
		public static function getUser() {
			global $ardeUser;
			
			if( isset( $_COOKIE[ $ardeUser->settings[ 'cookie_prefix' ] ] ) ) {
				$user = self::getUserByRandom( $_COOKIE[ $ardeUser->settings[ 'cookie_prefix' ] ] );
				if( $user !== null ) return $user;
			}
			
			return new ArdeUserPublic();

		}
		
		public static function getRootSessionUser() {
			global $ardeUser;
			
			if( isset( $_COOKIE[ $ardeUser->settings[ 'cookie_prefix' ].'_root_session' ] ) ) {
				$dbUsers = new ArdeDbUsers( $ardeUser->db );
				$rnd = $dbUsers->getRootSessionRandom();
				if ( $rnd === $_COOKIE[ $ardeUser->settings[ 'cookie_prefix' ].'_root_session' ] ) return new ArdeUserRoot();
			}
			
			return null;
		}
		
		public static function getUserByRandom( $random ) {
			global $ardeUser;
			
			if( empty( $random ) ) return null;
			
			$dbUsers = new ArdeDbUsers( $ardeUser->db );
			$dbUser = $dbUsers->getUserByRandom( $random );
			
			if( $dbUser === null ) return null;
			
			if( $dbUser->id == self::USER_ROOT ) return new ArdeUserRoot();
			else return new ArdeUser();
		}
		
		public static function getUserByUnPass( $username, $password ) {
			return self::getRootUserByUnPass( $username, $password );
		}
		
		public static function getRawRootUserByUnPass( $username, $password ) {
			global $ardeBase;
			if( $username == $ardeBase->settings[ 'root_username' ] && $password == $ardeBase->settings[ 'root_password' ] ) {
				$o = new ArdeUserRoot();
				return $o;
			}
			
			return null;
		}
		
		public static function getRootUserByUnPass( $username, $password ) {
			$o = self::getRawRootUserByUnPass( $username, $password );
			if( $o === null ) return null;
			$o->getInfoFromDb();
			return $o;
		}

		public function getInfoFromDb() {
			global $ardeUser;
			$dbUsers = new ArdeDbUsers( $ardeUser->db );
			$this->initWithDbUser( $dbUsers->getUserById( $this->id ) );
		}
		
		public function initWithDbUser( ArdeDbUser $dbUser ) {
			$this->id = $dbUser->id;
			$this->username = $dbUser->username;
			$this->random = $dbUser->random;
			$this->randomExpires = $dbUser->randomExpires;
		}
		
		public static function makePassiveRandomString() {
			global $ardeUser;
			return microtime().mt_rand( 0, 0x7fffffff ).$ardeUser->settings[ 'salt' ];
		}
		
		public static function makePassiveRandom() {
			return md5( self::makePassiveRandomString() );
		}
		
		public static function makeActiveRandom( $soft = false ) {
			global $ardeUser;
			$dbUsers = new ArdeDbUsers( $ardeUser->db );
			$lastRandom = $dbUsers->getLastRandom( $soft );
			$randomStr = $lastRandom.self::makePassiveRandomString();
			$rnd = md5( $randomStr );
			if( $lastRandom != '' ) $dbUsers->setLastRandom( $rnd );
			return $rnd;
		}
		
		public function renewRandom( $expires ) {
			global $ardeUser;
			
			$newRandom = self::makeActiveRandom();
			
			$dbUsers = new ArdeDbUsers( $ardeUser->db );
			$dbUsers->setRandom( $this->id, $newRandom, $expires );
			
			$this->random = $newRandom;
			$this->randomExpires = $expires;
			
		}
		
		public function removeRandom() {
			global $ardeUser;
			$dbUsers = new ArdeDbUsers( $ardeUser->db );
			$dbUsers->setRandom( $this->id, '', 0 );
		}
		
		public static function install( $overwrite ) {
			global $ardeUser;
			
			$dbUsers = new ArdeDbUsers( $ardeUser->db );
			$dbUsers->install( self::makePassiveRandom(), $overwrite );
			$dbUsers->addUser( self::USER_ROOT );
		}
		
		public static function uninstall() {
			global $ardeUser;
			
			$dbUsers = new ArdeDbUsers( $ardeUser->db );
			$dbUsers->uninstall();
		}
		
		public function hasPermission( $id, $subId = 0 ) {
			return $this->data->get( $id, $subId );
		}
	}
	
	class ArdeUserPublic extends ArdeUser {
		
		public function __construct() {
			global $ardeUser;
			$this->id = self::USER_PUBLIC;
			$this->username = 'guest';

		}

	}
	
	class ArdeUserRoot extends ArdeUser {
		
		const SESSION_LENGTH = 900;
		
		public function __construct() {
			global $ardeUser;
			$this->id = self::USER_ROOT;
			$this->username = $ardeUser->settings[ 'root_username' ];
		}
		
		public function initWithDbUser( ArdeDbUser $dbUser ) {
			$this->random = $dbUser->random;
			$this->randomExpires = $dbUser->randomExpires;
		}
		
		public function makeSessionRandom() {
			global $ardeUser;
			$newRandom = self::makeActiveRandom( true );
			$dbUsers = new ArdeDbUsers( $ardeUser->db );
			$dbUsers->setRootSessionRandom( $newRandom, self::SESSION_LENGTH );
			return $newRandom;
		}
		
		public function terminateSession() {
			global $ardeUser;
			$dbUsers = new ArdeDbUsers( $ardeUser->db );
			$dbUsers->uninstallRootSession();
			
		}
		
		public function hasPermission( $id, $subId = 0 ) {
			return true;
		}
		
	}
?>